#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDecoratorValues = exports.getDecorators = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const lodash_1 = require("lodash");
const yargs_1 = __importDefault(require("yargs"));
const glob_1 = __importDefault(require("glob"));
const typescript_1 = __importDefault(require("typescript"));
function getBasePath() {
    const currentModuleRelativePath = lodash_1.get(require, 'main.filename');
    if (!currentModuleRelativePath)
        throw new Error('Unable to locate module path');
    const currentModuleAbsolutePath = path_1.default.dirname(currentModuleRelativePath);
    const pathComponents = currentModuleAbsolutePath.split('node_modules');
    const basePath = pathComponents.slice(0, pathComponents.length - 1).join('node_modules');
    return basePath;
}
function getConfig() {
    const args = yargs_1.default(process.argv).argv;
    const configFileName = args && args.config ? args.config : 'dsapidoc.json';
    const configFile = getBasePath() + configFileName;
    const config = JSON.parse(fs_1.default.readFileSync(configFile).toString());
    validateConfig(config);
    return config;
}
function getCompilerOptions() {
    const configFile = getBasePath() + 'tsconfig.json';
    const { compilerOptions } = JSON.parse(fs_1.default.readFileSync(configFile).toString());
    if (!compilerOptions)
        throw new Error('tsconfig.json/compilerOptions not found');
    return compilerOptions;
}
function validateConfig(config) {
    const { include, exclude } = config;
    if (!Array.isArray(include) || !Array.isArray(exclude))
        throw new Error('Malformed config file');
    [...include, ...exclude].forEach(e => {
        if (!/^\.\//.test(e))
            throw new Error('Paths must be relative to project root');
    });
}
function getFiles() {
    const { exclude, include } = getConfig();
    const ignore = exclude.map(item => getBasePath() + item.replace(/^\.\//, ''));
    const toInclude = include.reduce((all, item) => {
        const path = getBasePath() + item.replace(/^\.\//, '');
        const paths = glob_1.default.sync(path, { ignore });
        return [...new Set([...all, ...paths])];
    }, []);
    const filtered = toInclude.filter(e => e.endsWith('.ts') && !e.endsWith('.d.ts'));
    return filtered;
}
function run() {
    const files = getFiles();
    const compilerOptions = getCompilerOptions();
    const program = typescript_1.default.createProgram(files, compilerOptions);
    const nodes = [];
    program.getSourceFiles().forEach(sf => {
        typescript_1.default.forEachChild(sf, node => {
            /**
             * If we declare a namespace within a module, like we do in `tsoaTestModule.d.ts`,
             * we need to explicitly get the children of the module declaration
             * (`declare module 'tsoaTest'`) - which are the moduleBlock statements,
             * because otherwise our type resolver cannot iterate over namespaces defined in that module.
             */
            if (typescript_1.default.isModuleDeclaration(node)) {
                /**
                 * For some reason unknown to me, TS resolves both `declare module` and `namespace` to
                 * the same kind (`ModuleDeclaration`). In order to figure out whether it's one or the other,
                 * we check the node flags. They tell us whether it is a namespace or not.
                 */
                // eslint-disable-next-line no-bitwise
                if ((node.flags & typescript_1.default.NodeFlags.Namespace) === 0 && node.body && typescript_1.default.isModuleBlock(node.body)) {
                    node.body.statements.forEach(statement => {
                        nodes.push(statement);
                    });
                    return;
                }
            }
            nodes.push(node);
        });
    });
    for (const node of nodes) {
        if (node.kind === typescript_1.default.SyntaxKind.ClassDeclaration && getDecorators(node, 'Controller').length > 0) {
            // for non string types consult tsoa
            const decorator = getDecorators(node, 'Controller')[0];
            const value = getDecoratorValues(decorator);
            console.log(value);
        }
    }
}
run();
function getDecorators(node, name) {
    const decorators = node.decorators;
    if (!decorators || !decorators.length) {
        return [];
    }
    return decorators
        .filter((decorator) => decorator.expression.escapedText === name);
}
exports.getDecorators = getDecorators;
// consult tsoa for non string values
function getDecoratorValues(decorator) {
    if (decorator && decorator.expression && Array.isArray(decorator.expression.arguments)) {
        return decorator.expression.arguments.map((e) => e.text);
    }
    return [];
}
exports.getDecoratorValues = getDecoratorValues;
;
